From 3b4904b79d9d14ddb82aa2a3285fdf4aef69b6df Mon Sep 17 00:00:00 2001
From: neirac <cneirabustos@gmail.com>
Date: Wed, 6 Oct 2021 17:37:04 +0000
Subject: [PATCH 3/3] fixup

---
 usr/src/uts/common/io/viocons/viocons.c | 28 ++++++++++++++++++++-----
 1 file changed, 23 insertions(+), 5 deletions(-)

diff --git a/usr/src/uts/common/io/viocons/viocons.c b/usr/src/uts/common/io/viocons/viocons.c
index b4298c48ca..4b9746eb7a 100644
--- a/usr/src/uts/common/io/viocons/viocons.c
+++ b/usr/src/uts/common/io/viocons/viocons.c
@@ -53,6 +53,7 @@
 #include "viocons.h"
 
 
+static int viocons_quiesce(dev_info_t *);
 static int viocons_attach(dev_info_t *, ddi_attach_cmd_t);
 static int viocons_detach(dev_info_t *, ddi_detach_cmd_t);
 
@@ -85,11 +86,11 @@ static struct dev_ops viocons_dev_ops = {
 
 	.devo_attach =			viocons_attach,
 	.devo_detach =			viocons_detach,
-	.devo_quiesce =			NULL,
+	.devo_quiesce =			viocons_quiesce,
 
 	.devo_cb_ops =			&viocons_cb_ops,
 
-	.devo_getinfo =			NULL,
+	.devo_getinfo =			ddi_no_info,
 	.devo_identify =		nulldev,
 	.devo_probe =			nulldev,
 	.devo_reset =			nodev,
@@ -107,6 +108,17 @@ static struct modlinkage viocons_modlinkage = {
 	.ml_rev =			MODREV_1,
 	.ml_linkage =			{ &viocons_modldrv, NULL }
 };
+static int
+viocons_quiesce(dev_info_t *dip)
+{
+	viocons_t *viocons;
+
+	if ((viocons = ddi_get_driver_private(dip)) == NULL) {
+		return (DDI_FAILURE);
+	}
+
+	return (virtio_quiesce(viocons->vcons_virtio));
+}
 
 static int
 viocons_attach(dev_info_t *dip, ddi_attach_cmd_t cmd)
@@ -114,6 +126,7 @@ viocons_attach(dev_info_t *dip, ddi_attach_cmd_t cmd)
 	int ret;
 	viocons_t *vcons;
 	virtio_t *vio;
+	boolean_t did_mutex = B_FALSE;
 
 	if (cmd != DDI_ATTACH) {
 		return (DDI_FAILURE);
@@ -121,21 +134,26 @@ viocons_attach(dev_info_t *dip, ddi_attach_cmd_t cmd)
 
 	if ((vio = virtio_init(dip, VIRTIO_CONSOLE_WANTED_FEATURES, B_TRUE)) ==
 	    NULL) {
+		dev_err(dip, CE_WARN, "failed to start Virtio init");
 		return (DDI_FAILURE);
 	}
 
 	vcons = kmem_zalloc(sizeof (*vcons), KM_SLEEP);
 	vcons->vcons_dip = dip;
 	vcons->vcons_virtio = vio;
-
+	ddi_set_driver_private(dip, vio);
 
 	if (virtio_init_complete(vio, 0) !=
 	    DDI_SUCCESS) {
 		dev_err(dip, CE_WARN, "failed to complete Virtio init");
 		goto fail;
 	}
-
-	mutex_exit(&vcons->vcons_mutex);
+	/*
+	 * Enable interrupts now so that we can request the device identity.
+	 */
+	if (virtio_interrupts_enable(vio) != DDI_SUCCESS) {
+		goto fail;
+	}
 
 	return (DDI_SUCCESS);
 
-- 
2.33.0

